#include "../lib.h"

#ifdef TARGET_LINUX
#include <stdio.h>
#endif

void fail_start_trace(void) {}
void fail_stop_trace(void) {}
void fail_marker_positive(void) {}
void fail_marker_negative(void) {}
void fail_marker_detected(void) {}
void print(const char *msg) { PRINT("[C]       %s", msg); }

int wasm_module(void);

MAIN {
  int retval = wasm_module();
  PRINT_SUCCESS("wasm_module returned %d.\n", retval);
  RET(retval);
}
